import { WDAG } from '../supporting/wdag.js';

class Author {
  constructor() {
    this.posts = new Map();
    this.reputation = 0;
  }
}

class PostVertex {
  constructor(id, author, stake, content, citations) {
    this.id = id;
    this.author = author;
    this.content = content;
    this.stake = stake;
    this.citations = citations;
    this.reputation = 0;
  }
}

export class ForumView {
  constructor() {
    this.reputations = new Map();
    this.posts = new WDAG();
    this.authors = new Map();
  }

  getReputation(id) {
    return this.reputations.get(id);
  }

  setReputation(id, reputation) {
    this.reputations.set(id, reputation);
  }

  incrementReputation(publicKey, increment, _reason) {
    const reputation = this.getReputation(publicKey) || 0;
    return this.reputations.set(publicKey, reputation + increment);
  }

  getOrInitializeAuthor(authorId) {
    let author = this.authors.get(authorId);
    if (!author) {
      author = new Author(authorId);
      this.authors.set(authorId, author);
    }
    return author;
  }

  addPost(authorId, postId, postContent, stake) {
    const { citations = [], content } = postContent;
    const author = this.getOrInitializeAuthor(authorId);
    const postVertex = new PostVertex(postId, author, stake, content, citations);
    this.posts.addVertex(postId, postVertex);
    for (const { postId: citedPostId, weight } of citations) {
      this.posts.addEdge('citation', postId, citedPostId, weight);
    }
  }

  getPost(postId) {
    return this.posts.getVertexData(postId);
  }

  getPosts() {
    return this.posts.getVertices();
  }
}
